/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * atom_graphnode.cpp
 *
 *  Created on: Oct 19, 2009
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include <iostream>

#include "atom_graphnode.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"

/** Constructor of class GraphNode.
 */
GraphNode::GraphNode()
{
};

/** Destructor of class GraphNode.
 */
GraphNode::~GraphNode()
{
};

/** Output graph info of this atom.
 * \param *out output stream
 */
void GraphNode::OutputGraphInfo() const
{
  LOG(2, "Atom " << getName() << " is " << ((SeparationVertex) ? "a" : "not a") << " separation vertex, components are " << *this);
  LOG(3, " with Lowpoint " << LowpointNr << " and Graph Nr. " << GraphNr << ".");
};

/** Output a list of flags, stating whether the bond was visited or not.
 * Note, we make use of the last entry of the ComponentNr always being -1 if allocated.
 * \param *out output stream for debugging
 */
void GraphNode::OutputComponentNumber(ostream *out) const
{
  *out << *this;
};

ostream &operator<<(ostream &out, const GraphNode &a)
{
  if (a.ComponentNr != NULL) {
    for (int i=0; a.ComponentNr[i] != -1; i++)
      out << (i!=0 ? "," : "") << a.ComponentNr[i];
  }
  return out;
}
